@@ -15,6 +15,7 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include "button-widget.h"
 #include "panel-types.h"
+#include "panel-background.h"
 
 G_BEGIN_DECLS
 
@@ -77,9 +78,7 @@ struct _PanelWidget
 
 	int             thick;
 
-	PanelBackType   back_type;
-	char           *back_pixmap;
-	PanelColor      back_color;
+	PanelBackground background;
 
 	GtkWidget      *master_widget;
 	
@@ -92,19 +91,6 @@ struct _PanelWidget
 	
 	GdkEventKey    *key_event;
 
-	GdkPixbuf      *backpix;           /* background pixmap unscaled */
-	GdkPixbuf      *backpix_scaled;    /* pixmap scaled to panel size */
-
-	GdkPixbuf      *desktop_image;     /* whats on the desktop */
-	GdkPixbuf      *background_image;  /* what to draw on the panel */
-	GdkPixmap      *background_pixmap; /* pixmap to give to applets */
-
-	gulong          background_changed_signal_handler;
-
-	guint           fit_pixmap_bg : 1;     /* fit pixmap while keeping ratio */
-	guint           stretch_pixmap_bg : 1; /* stretch pixmap to the size of the panel */
-	guint           rotate_pixmap_bg : 1;  /* rotate pixmap on vertical panels */
-
 	guint           packed : 1;
 
 	guint           inhibit_draw : 1;
@@ -136,12 +122,12 @@ struct _PanelWidgetClass
 
 GType		panel_widget_get_type		(void) G_GNUC_CONST;
 
-GtkWidget *	panel_widget_new		(gchar *panel_id,
+GtkWidget *	panel_widget_new		(const char *panel_id,
 						 gboolean packed,
 						 GtkOrientation orient,
 						 int sz,
-						 PanelBackType back_type,
-						 char *back_pixmap,
+						 PanelBackgroundType back_type,
+						 const char *back_pixmap,
 						 gboolean fit_pixmap_bg,
 						 gboolean stretch_pixmap_bg,
 						 gboolean rotate_pixmap_bg,
@@ -189,8 +175,8 @@ void		panel_widget_applet_drag_end_no_grab(PanelWidget *panel);
 void		panel_widget_change_params	(PanelWidget *panel,
 						 GtkOrientation orient,
 						 int sz,
-						 PanelBackType back_type,
-						 char *pixmap_name,
+						 PanelBackgroundType back_type,
+						 const char *pixmap_name,
 						 gboolean fit_pixmap_bg,
 						 gboolean stretch_pixmap_bg,
 						 gboolean rotate_pixmap_bg,
